## Explore Stamatoyannopoulos ENCODE metadata

library(Rsamtools)
library(parallel)
mcsapply <- 
    if (!is.null(getFunction("mclapply", mustFind=FALSE))) {
        function(...) simplify2array(mclapply(...))
    } else {
        sapply
    }

## metadata
## https://spreadsheets.google.com/pub?key=0Aut0bk4UkkZCdFZ4ZEhIMnIxREV0dDRnUjlYcXp1c1E
## see http://encodeproject.org/ENCODE/dataSummary.html

fl <- system.file("extdata", "HumanExperiments.2011-01-07.csv",
                  package="SequenceAnalysisData")
csv <- read.csv(fl)

ridx <- with(csv, {
    Data_Type=="ChIP-seq" & PI=="Stamatoyannopoulos" &
    Assembly=="hg19" & grepl("^(Antibody=CTCF|Control)",
      Experimental_Factors)
})
cidx <- !names(csv) %in% c("Data_Type", "Treatment", "Lab", "PI",
                           "Status", "Date_Unrestricted", "Assembly")
csv1 <- csv[ridx, cidx]

ridx <- with(csv1, {
    Cell_Type %in% Cell_Type[grepl("Antibody", Experimental_Factors)]
})

csv1 <- csv1[ridx,]
o <- with(csv1, order(Cell_Type, Experimental_Factors))
csv1[o,]

## series matrix

url <- "ftp://ftp.ncbi.nih.gov/pub/geo/DATA/SeriesMatrix/GSE30263/GSE30263_series_matrix.txt.gz"
dest <- tempfile()
cs


## BAM
fls <- list.files(system.file("bigdata", "chipseq-stam",
                              package="SequenceAnalysisData"),
                  pattern="bam$", full=TRUE)
bf <- BamFile(fls[1])
scanBamHeader(bf)                       # no SQ
countBam(bf)                            # 12.2 M reads

which <- GRanges("chr12", IRanges(1, 133851895))
param <- ScanBamParam(which=which, what=character())
aln <- readBamGappedAlignments(bf, param=param) # 8MB; 606k reads
table(cigar(aln))                       # 36M

aln <- keepSeqlevels(aln, "chr12")
cvg <- coverage(aln)

isles <- mcsapply(1:20, function(i, cvg) length(slice(cvg, i)[[1]]),
                  cvg)
plot(log(isles)~log(seq_along(isles)), type="b")
