## Create SummarizedExperiment 
## rowData : GRanges of "master" peak regions
## assays  : matrices of 
##           tag counts, p-values, and no. summits in each "master"
##           peak
## colData : data related to genome build, replicate, etc. 

peakHalfWidth <- 20

library(rtracklayer)
library(GenomicRanges)
library(parallel)
mclapply <-
    if (!exists("mclapply", getNamespace("parallel"))) {
        lapply
    } else function(...) {
        parallel::mclapply(..., mc.cores=detectCores())
    }
mcsapply <- function(...) simplify2array(mclapply(...))

bedFiles <- list.files("./", ".*_summits.bed$", recursive=TRUE)
bedFiles <- bedFiles[!grepl("NOcontrol", bedFiles)]
xlsFiles <- sub("_summits.bed$", "_peaks.xls", bedFiles)
stopifnot(all(sapply(bedFiles, file.exists)))
stopifnot(all(sapply(xlsFiles, file.exists)))

## rowData :
lst <- Map(function(bed, xls) {
    gr <- as(import(bed), "GRanges")
    pvalue <- read.delim(xls, comment.char="#")$"X.10.log10.pvalue."
    values(gr) <- append(values(gr), DataFrame(pvalue=pvalue))
    gr
}, bedFiles, xlsFiles)

pksummit <- lst[[1]]
values(pksummit) <- NULL
for (elt in lst[-1]) {
    values(elt) <- NULL
    pksummit <- c(pksummit, elt)
}

## pksummit <- do.call(c, lst)
pkrange <- resize(pksummit, peakHalfWidth, "center")
rowData <- reduce(pkrange)

## assays :
counts <- mcsapply(lst, function(x, rowData) {
    olap <- findOverlaps(x, rowData) 
    gpcts <- split(values(x)[["score"]][queryHits(olap)],
                   subjectHits(olap))
    meancts <- do.call(c, lapply(gpcts, mean))
    counts <- rep(0, length(rowData))
    counts[subjectHits(olap)] <- meancts
    counts 
}, rowData)

# a lot of repeated code here...
pvalues <- mcsapply(lst, function(x, rowData) {
    olap <- findOverlaps(x, rowData) 
    gppval <- split(values(x)[["pvalue"]][queryHits(olap)],
                    subjectHits(olap))
    meanpval <- do.call(c, lapply(gppval, mean))
    pvalues <- rep(0, length(rowData))
    pvalues[subjectHits(olap)] <- meanpval 
    pvalues 
}, rowData)

summitPerPeak <- mcsapply(lst, countOverlaps, query=rowData)

## colData
colData <- DataFrame(
    SummitFile=basename(bedFiles),
    PeaksFile=basename(xlsFiles),
    Genome=rep("hg19", length(lst)), 
    CellLine=sub("_summits.bed$", "", basename(bedFiles)),
    Replicate=factor(sub("results_hg19_rep([12]).*", "\\1", bedFiles),
                     levels=c("1", "2")))
names(lst) <- rownames(colData) <-
    with(colData, sprintf("%s_%s", CellLine, Replicate))

stam <- SummarizedExperiment(rowData=rowData, colData=colData,
    assays=SimpleList(counts=counts, pvalues=pvalues, 
    summitPerPeak=summitPerPeak))

save(stam, file="stam.Rda")

## cidx = with(colData(stam), CellLine %in% CellLine[Replicate==2])
## x = varFilter(assays(stam[,cidx])[["counts"]], var.cutoff=.8)
## x1 = mean(colSums(x)) * x / colSums(x)[col(x)]
## heatmap(x1, Rowv=NA)
